#include "libsandbox.hpp"
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <pwd.h>
#include <fcntl.h>
#include <errno.h>

using namespace cv;
using namespace std;

#define BUFF_SIZE 1024
//argv[1] = adresse IP d'un serveur
//argv[2] = le numéro de port sur lequel tourne le service
/* AF_INET= ipv4, SOCK_STREAM corresponds à TCP*/

char buff_e[BUFF_SIZE];
int idRequest = 0;
int matrixSize = sizeof(matrixPayload_t) - sizeof(unsigned char *);
void Sandbox::matrixToPayload(Mat m, matrixPayload_t *p)
{
	p->width = m.cols;
	p->height = m.rows;
	p->type = m.depth();
	p->nbCanal = m.channels();
	p->data = m.data;
}

void Sandbox::payloadToMatrix(Mat *m, matrixPayload_t p)
{
	int typeOpenCV = 8 * (p.nbCanal - 1) + p.type;
	*m = Mat(Size(p.width, p.height), typeOpenCV, (void *)p.data, Mat::AUTO_STEP).clone();
}

int Sandbox::calculateSizeMatrix(matrixPayload_t payload)
{
	static char bytes[] = {1, 1, 2, 2, 4, 4, 8};
	return payload.height * payload.width * payload.nbCanal * bytes[payload.type];
}

void Sandbox::readData(char *buff, int length)
{
	int byteToRead = length;
	while (byteToRead > 0)
	{
		int nb = read(socketFD, buff + length - byteToRead, byteToRead);
		if (nb < 1) //end of communication
			exit(0);
		byteToRead -= nb;
	}
}

void Sandbox::action(int idAction, Mat input, Mat *output)
{
	//send request
	request_t request;
	request.id = idRequest++;
	request.idAction = idAction;
	matrixPayload_t payload;
	if (!input.empty())
	{
		matrixToPayload(input, &payload);
		request.lengthPayload = calculateSizeMatrix(payload);
	}
	else
		request.lengthPayload = 0;
	write(socketFD, &request, sizeof(request));
	write(socketFD, &payload, matrixSize);
	if (request.lengthPayload)
		write(socketFD, payload.data, request.lengthPayload);
	//receive answer
	memset(buff_e, 0, BUFF_SIZE);
	readData(buff_e, sizeof(answer_t));
	answer_t *ans = (answer_t *)buff_e;
	//check magig
	if (ans->magic == MAGIC_ANS)
	{
		matrixPayload_t matrix;
		readData((char *)&matrix, matrixSize);

		if (ans->lengthPayload != 0)
		{
			//receive matrix
			//don't forget to free the memory
			char *buff_p = new char[ans->lengthPayload];
			readData(buff_p, ans->lengthPayload);
			matrix.data = (unsigned char *)buff_p;
			payloadToMatrix(output, matrix);
		}
	}
}

void Sandbox::showImage(Mat image)
{
	action(0, image, NULL);
}
void Sandbox::getRGBFrame(Mat *frame)
{
	action(1, Mat(), frame);
}
void Sandbox::getDepthFrame(Mat *frame)
{
	action(2, Mat(), frame);
}
Sandbox::Sandbox(){}
Sandbox::Sandbox(char *ip, int port)
{
	struct sockaddr_in addr_internet;

	/* initialisation socket */
	if ((socketFD = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		perror("socket");
		exit(1);
	}

	addr_internet.sin_family = AF_INET;
	addr_internet.sin_port = (in_port_t)htons(port);
	addr_internet.sin_addr.s_addr = inet_addr(ip);

	if ((connect(socketFD, (struct sockaddr *)&addr_internet, sizeof(addr_internet)) < 0))
		perror("connect");
}