#!/usr/bin/env python
# coding: utf-8
from __future__ import division
import socket
import struct
import random
from datetime import datetime

import math
import sys
import numpy as np
import cv2
import array
import open3d


class Sandbox:
    NB_BYTE_HANDLE = 32

    def __init__(self, host, port):
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.settimeout(15)
        try:
            self.sock.connect((host, int(port)))
        except socket.error:
            sys.exit("Cannot connect to %s:%s" % (host, port))

    def __send_request(self, id):
        # Send get depth frame
        struct_req = struct.pack("<IIBI", 0x76767676, 0, id, 0)
        struct_req += struct.pack("<IIBB", 0, 0, 0, 0)
        self.sock.sendall(struct_req)
        receive = ""
        while len(receive) < 16:
            receive += self.sock.recv(16 - len(receive))
        magic, error, id, length = struct.unpack("<IIII", receive)
        #Get depth frame
        receive = ""
        while len(receive) < 10:
            receive += self.sock.recv(10 - len(receive))
        
        return receive



    def show_image(self):
        self.__send_request(0)
        print("TODO: Show image")

    
    def get_rgb_frame(self):
        receive = self.__send_request(1)
        w, h, typeM, nbCanal = struct.unpack("<IIBB", receive)

        receive = ""
        while len(receive) < length:
            receive += self.sock.recv(length - len(receive))
        z = array.array('H', receive)
        z2 = np.array(z)
        return z2

    def get_depth_frame(self):
        # Send get depth frame
        receive = self.__send_request(2)
        w, h, typeM, nbCanal = struct.unpack("<IIBB", receive)

        receive = ""
        while len(receive) < length:
            receive += self.sock.recv(length - len(receive))
        z = array.array('H', receive)
        z2 = np.array(z)
        return z2

    def end_connection(self):
        """
        This simply close the socket
        :return:
        """
        self.sock.close()
