using namespace std;

#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <netinet/in.h>
#include <string.h>
#include "APISandbox.hpp"
#include "log.h"
#include <opencv2/opencv.hpp>
#include "./libSandbox/sandbox.h"
#include <pthread.h>
using namespace cv;
using namespace std;

#define BUFF_SIZE 1024

#define CMD_READ 0x0
#define LENGTH_HEADER 13 //length of: magic + lengthpayload + id + idAction
#define MTU_MAX 1024
#define IP_PORT 1242
int matrixSize = sizeof(matrixPayload_t) - sizeof(unsigned char *);
Sandbox *sandbox;

void matrixToPayload(Mat m, matrixPayload_t *p)
{
    p->width = m.cols;
    p->height = m.rows;
    p->type = m.depth();
    p->nbCanal = m.channels();
    p->data = m.data;
}

void payloadToMatrix(Mat *m, matrixPayload_t p)
{
    int typeOpenCV = 8 * (p.nbCanal - 1) + p.type;
    cout << typeOpenCV << endl;
    *m = Mat(Size(p.width, p.height), typeOpenCV, (void *)p.data, Mat::AUTO_STEP);
}

int calculateSizeMatrix(matrixPayload_t payload)
{
    static char bytes[] = {1, 1, 2, 2, 4, 4, 8};
    return payload.height * payload.width * payload.nbCanal * bytes[payload.type];
}

void sendAnswer(request_t *req, matrixPayload_t *payload, int socket)
{
    answer_t ans;
    matrixPayload_t payloadAnswer;
    ans.id = req->id;
    cv::Mat matrix;
    switch (req->idAction)
    {
    case 0: //show
        mylog(LOG_DEBUG, "Request: show image\n");
        if (req->lengthPayload)
        {
            payloadToMatrix(&matrix, *payload);
            sandbox->showImage(matrix);
            waitKey(1);
        }
        else
        {
            mylog(LOG_ERROR, "No matrix found\n");
            ans.error = -1;
        }
        break;
    case 1:
        mylog(LOG_DEBUG, "Request: get RGB frame\n");
        sandbox->getRGBFrame().copyTo(matrix);
        matrixToPayload(matrix, &payloadAnswer);
        ans.lengthPayload = calculateSizeMatrix(payloadAnswer);
        break;
    case 2:
        mylog(LOG_DEBUG, "Request: get DEPTH frame\n");
        sandbox->getDepthFrame().copyTo(matrix);
        matrixToPayload(matrix, &payloadAnswer);
        ans.lengthPayload = calculateSizeMatrix(payloadAnswer);
        break;
    }

    write(socket, &ans, sizeof(answer_t));
    //send the matrix without the pointer
    write(socket, &payloadAnswer, matrixSize);
    if (ans.lengthPayload)
        write(socket, payloadAnswer.data, ans.lengthPayload);
    mylog(LOG_DEBUG, "Answer sent\n");
}

void readData(char *buff, int length, int socket)
{
    int byteToRead = length;
    while (byteToRead > 0)
    {
        int nb = read(socket, buff + length - byteToRead, byteToRead);
        //end of communication
        if (nb < 1)
        {
            pthread_exit(NULL);
        }

        byteToRead -= nb;
    }
}

/*---------------------------------------------------------------------------*-
   childServer()
  -----------------------------------------------------------------------------
   Description    : Code of the child server
-*---------------------------------------------------------------------------*/
void *childServer(void *sock)
{
    int new_socket = *(int *)sock;
    mylog(LOG_DEBUG, "New client\n");
    char buff_e[BUFF_SIZE] = {0};

    while (1)
    {
        memset(buff_e, 0, BUFF_SIZE);
        readData(buff_e, sizeof(request_t), new_socket);
        request_t *req = (request_t *)buff_e;
        mylog(LOG_DEBUG, "Message received\n");
        //check magig
        if (req->magic == MAGIC_REQ)
        {
            matrixPayload_t matrix;
            readData((char *)&matrix, matrixSize, new_socket);
            mylog(LOG_DEBUG, "payload lu\n");
            char *buff_p = new char[req->lengthPayload];
            if (req->lengthPayload != 0)
            {
                //receive matrix
                readData(buff_p, req->lengthPayload, new_socket);
                matrix.data = (unsigned char *)buff_p;
            }
            sendAnswer(req, &matrix, new_socket);
            free(buff_p);
        }
    }
    mylog(LOG_DEBUG, "Fin de connexion\n");
    exit(0);
}

/*---------------------------------------------------------------------------*-
   main()
  -----------------------------------------------------------------------------
   Description :  Main server (mutli-client)
-*---------------------------------------------------------------------------*/
void APISandbox()
{

    mylog(LOG_DEBUG, "Sandbox starting...\n");
    Sandbox sand;
    sandbox = &sand;
    int s;
    struct sockaddr_in addr_internet;
    struct sockaddr_in addr_client;

    if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
    {
        mylog(LOG_ERROR, "Socket\n");
        exit(1);
    }

    addr_internet.sin_family = AF_INET;
    addr_internet.sin_port = (in_port_t)htons(IP_PORT);
    addr_internet.sin_addr.s_addr = htonl(INADDR_ANY);

    if ((bind(s, (struct sockaddr *)&addr_internet, sizeof(addr_internet))) < 0)
    {
        mylog(LOG_ERROR, "Bind\n");
        exit(1);
    }

    if (listen(s, 10) < 0)
    {
        mylog(LOG_ERROR, "Listen\n");
        exit(1);
    }
    mylog(LOG_DEBUG, "Server is ready\n");
    socklen_t len_client;
    int new_socket;
    while (1)
    {
        pthread_t thread_id; //we don't use the value of the variable
        //wait until connexion
        if ((new_socket = accept(s, (struct sockaddr *)&addr_client, &len_client)) < 0)
        {
            mylog(LOG_ERROR, "Accept\n");
        }
        if (pthread_create(&thread_id, NULL, childServer, (void *)&new_socket) != 0)
        {
            mylog(LOG_ERROR, "Could not create thread\n");
            return;
        }
    }
}

int main()
{
    APISandbox();
    return 0;
}
