#include <algorithm>
#include "borderedit.h"

using namespace std;
using namespace cv;
Mat BorderEdit::frameImage;

void BorderEdit::drawSquare(cv::Point *p, int n)
{
    Mat imageCopy = frameImage.clone();
    polylines(imageCopy, &p, &n, 1, true, Scalar(0, 255, 0), 1, LINE_AA);
    imshow(wndname, imageCopy);
}
int BorderEdit::findPoints(int x, int y, vector<Point> &posSandbox)
{
    for (int i = 0; i < (int)posSandbox.size(); i++)
    {
        //check if the point is clicked
        if ((x >= (posSandbox[i].x - margeClick)) && (x <= (posSandbox[i].x + margeClick)) && (y >= (posSandbox[i].y - margeClick)) && (y <= (posSandbox[i].y + margeClick)))
            return i;
    }
    return -1; //not found
}

void BorderEdit::mouseHandler(int event, int x, int y, int, void *param)
{
    static int selectedPoint = -1;
    vector<Point> &posSandbox = *((vector<Point> *)param);
    switch (event)
    {
    case CV_EVENT_LBUTTONDOWN: //left button press
        selectedPoint = BorderEdit::findPoints(x, y, posSandbox);
        break;

    case CV_EVENT_LBUTTONUP: //left mouse button release
        if (selectedPoint != -1)
            selectedPoint = -1;
        break;

    case CV_EVENT_MOUSEMOVE:
        /* draw a rectangle*/
        if (selectedPoint != -1)
        {
            posSandbox[selectedPoint].x = x;
            posSandbox[selectedPoint].y = y;
            BorderEdit::drawSquare(&posSandbox[0], (int)posSandbox.size());
            Rect r = boundingRect(posSandbox);
            double f = (double)r.width / (double)r.height;
            cout << 4.0 / 3.0 << " == " << f << endl;
        }
        break;
    }
}

void BorderEdit::edit(Mat frame, vector<Point> *posSandbox)
{
    namedWindow(wndname, CV_WINDOW_AUTOSIZE);
    frame.copyTo(frameImage);
    setMouseCallback(wndname, BorderEdit::mouseHandler, (void *)posSandbox);
    BorderEdit::drawSquare(&posSandbox->at(0), (int)posSandbox->size());
    waitKey(0);
}