#include <algorithm>
#include "borderfinder.h"

using namespace std;
using namespace cv;

double BorderFinder::angle(Point pt1, Point pt2, Point pt0)
{
    double dx1 = pt1.x - pt0.x;
    double dy1 = pt1.y - pt0.y;
    double dx2 = pt2.x - pt0.x;
    double dy2 = pt2.y - pt0.y;
    return (dx1 * dx2 + dy1 * dy2) / sqrt((dx1 * dx1 + dy1 * dy1) * (dx2 * dx2 + dy2 * dy2) + 1e-10);
}

// returns sequence of squares detected on the image.
bool BorderFinder::find(Rect &rect)
{
    vector<vector<Point>> rectDetect;

    Mat pyr, timg, gray0(frameImage.size(), CV_8U);

    // down-scale and upscale the image to filter out the noise
    pyrDown(frameImage, pyr, Size(frameImage.cols / 2, frameImage.rows / 2));
    pyrUp(pyr, timg, frameImage.size());
    vector<vector<Point>> contours;
    int c = 1;
    int ch[] = {c, 0};
    mixChannels(&timg, 1, &gray0, 1, ch, 1);
    gray0 = gray0 > 10;
    waitKey(0);
    findContours(gray0, contours, RETR_LIST, CHAIN_APPROX_SIMPLE);
    vector<Point> approx;
    // test each contour
    cout << "nb rect: " <<  contours.size() << endl;
    for (size_t i = 0; i < contours.size(); i++)
    {
        // approximate contour with accuracy proportional
        // to the contour perimeter
        approxPolyDP(contours[i], approx, arcLength(contours[i], true) * 0.02, true);
        if (approx.size() == 4 &&
            fabs(contourArea(approx)) > 1000 &&
            isContourConvex(approx))
        {
            double maxCosine = 0;

            for (int j = 2; j < 5; j++)
            {
                // find the maximum cosine of the angle between joint edges
                double cosine = fabs(angle(approx[j % 4], approx[j - 2], approx[j - 1]));
                maxCosine = MAX(maxCosine, cosine);
            }

            // if cosines of all angles are small
            // (all angles are ~90 degree) then write quandrange
            // vertices to resultant sequence
          //  if (maxCosine < 0.3)
           // {
                rect = rotateRect(boundingRect(approx), 90, 90);
                const Point *p = &approx[0];
                int n = (int)approx.size();
                polylines(frameImage, &p, &n, 1, true, Scalar(0, 0, 255), 3, LINE_AA);
                //return true;
            //}
        }
    }
    //imshow("ff", frameImage);
    //waitKey(0);
    return true;
}

Rect BorderFinder::rotateRect(Rect rect, int heightPercentage, int widthPercetange)
{
    int rwidth = rect.width;
    int rheight = rect.height;

    rect.width = round((rect.width * widthPercetange) / 100.0f);
    rect.height = round((rect.height * heightPercentage) / 100.0f);
    rect.x += (rwidth - rect.width) / 2;
    rect.y += (rheight - rect.height) / 2;

    return rect;
}

float BorderFinder::calculDistance(float point1[], float point2[])
{
    return sqrt(pow(point1[0] - point2[0], 2) +
                pow(point1[1] - point2[1], 2) +
                pow(point1[2] - point2[2], 2));
}

BorderFinder::BorderFinder(Mat frame)
{
    frame.copyTo(frameImage);
}
