#include "calibrate.h"
#include "camera.h"

using namespace cv;
using namespace std;

Calibrate::Calibrate()
{
}

Point2i Calibrate::rotatePixel(Point2i pixel)
{
    Mat tmp = (Mat_<Vec2f>(1, 1) << Vec2f(pixel.x, pixel.y));
    transform(tmp, tmp, matRotation);
    return Point2i(tmp.at<Vec2f>(0, 0));
}

Point2i Calibrate::transformationPixel(int i, int j, float z, Camera camera, Point3f beamer)
{
    //pixel to point 3d
    float coord[2] = {static_cast<float>(j), static_cast<float>(i)};
    Point3f p = camera.deprojectPixelToPoint(coord, z / 1000.0);
    //calcul vector point to beamer
    Vec3f dir(beamer.x - p.x, beamer.y - p.y, beamer.z - p.z);
    float prop = (0.99 - p.z) / dir[2];
    p.x += dir[0] * prop;
    p.y += dir[1] * prop;
    p.z += dir[2] * prop;
    //point 3d to pixel
    return camera.projectPointToPixel(p);
}

void Calibrate::transformationFrame(cv::Mat &src, cv::Mat &dst, Camera camera, Point3f beamer)
{
    int64_t t1 = getTickCount();
    //transformation on all pixel
    for (int i = 0; i < src.rows; i++)
    {
        for (int j = 0; j < src.cols; j++)
        {
            Point pixelIJ(j, i);
            Point pixel = transformationPixel(i, j, static_cast<float>(src.at<uint16_t>(pixelIJ)), camera, beamer);
            if (pixel.x < dst.cols && pixel.y < dst.rows && pixel.x >= 0 && pixel.y >= 0)
                dst.at<uint16_t>(pixel) = src.at<uint16_t>(pixelIJ);
        }
    }
    cout << "temps de calcul: " << (getTickCount() - t1) / getTickFrequency() << endl;

    warpAffine(dst, dst, matRotation, dst.size());
    // medianBlur(dst, dst, 3);
}

void Calibrate::transformationFrame(cv::Mat &depth, cv::Mat &src, cv::Mat &dst, Camera camera, Point3f beamer)
{
    int nbChannel = src.channels();
    //transformation on all pixel
    int64_t t1 = getTickCount();
    for (int i = 0; i < src.rows; i++)
    {
        for (int j = 0; j < src.cols; j++)
        {
            Point pixelIJ(j, i);
            Point pixel = transformationPixel(i, j, static_cast<float>(depth.at<uint16_t>(pixelIJ)), camera, beamer);
            if (pixel.x < dst.cols && pixel.y < dst.rows && pixel.x >= 0 && pixel.y >= 0)
            {
                if (nbChannel == 1)
                    dst.at<char>(pixel) = src.at<char>(pixelIJ);
                else if (nbChannel == 3)
                    dst.at<Vec3b>(pixel) = src.at<Vec3b>(pixelIJ);
            }
        }
    }
    cout << "temps de calcul: " << (getTickCount() - t1) / getTickFrequency() << endl;
    warpAffine(dst, dst, matRotation, dst.size());
    dilate(dst, dst, Mat(), Point(-1, -1), 2, 1, 1);
    erode(dst, dst, Mat(), Point(-1, -1), 2, 1, 1);
}
