#ifndef SANDBOX_H
#define SANDBOX_H

#include "camera.h"
#include "borderedit.h"
#include "borderfinder.h"
#include "calibrate.h"
#include "beamer.h"

class Sandbox
{

private:
    const char *SANDBOX_POSITION_FILE = "./backend/config/sandbox.dat";
    const char *CALIBRATE_DISTANCE_FILE = "./backend/config/distance.dat";
    const char *CALIBRATE_MATRIX_FILE = "./backend/config/matrixe.dat";
    const char CHAR_DELIM = ' ';
    const float SANDBOX_CONTOUR = 2.0f * (1020.0f + 770.0f);
    const float MARGE = 0.05 * SANDBOX_CONTOUR; // 5% of marge
    const int ESCAPE_CHAR = 27;
    char *defaultWindowsName = (char*) "Image";
    cv::Rect rectSandbox;
    Calibrate calibrate;
    Camera camera;
    Beamer beamer;
    void createWindowsFullScreen(char *windowName);
    void showImage(cv::Mat image, char *windowName);
    bool readSandboxPosition(std::vector<cv::Point> &rectPoints);
    void writeSandboxPosition(std::vector<cv::Point> rectPoints);
    double toDegrees(double radians);
    void sanboxBorder();
    void sandboxBorderLoad();
    bool configFilesFound();

public:
    Sandbox();
    cv::Mat getRGBFrame();
    cv::Mat getDepthFrame(); 
    void showImage(cv::Mat image);
    void exportToAsc(cv::Mat frame, char *fileName);
    void exportFrame(cv::Mat src, char *fileName);
    bool importFrame(char *fileName, cv::Mat &output);
};
#endif